"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDeleteModelRunner = exports.DeleteModelRunner = void 0;
var _store = require("./helpers/store");
class DeleteModelRunner {
  constructor(params) {
    this.taskId = params.taskId;
    this.context = params.context;
    this.response = params.response;
  }
  async execute(params) {
    const {
      input,
      isCloseToTimeout,
      isAborted
    } = params;
    const model = await this.getModel(input.modelId);
    let hasMoreItems = false;
    let lastDeletedId = input.lastDeletedId;
    do {
      if (isAborted()) {
        /**
         * If the task was aborted, we need to remove the task tag from the model.
         */
        await this.removeBeingDeleted(model);
        return this.response.aborted();
      } else if (isCloseToTimeout()) {
        return this.response.continue({
          ...input,
          lastDeletedId
        });
      }
      const where = {
        latest: true
      };
      if (lastDeletedId) {
        where.entryId_gte = lastDeletedId;
      }
      const {
        items,
        hasMoreItems: metaHasMoreItems
      } = await this.context.cms.storageOperations.entries.list(model, {
        limit: 1000,
        where,
        sort: ["entryId_ASC"]
      });
      for (const item of items) {
        try {
          await this.context.cms.deleteEntry(model, item.id, {
            permanently: true,
            force: true
          });
        } catch (ex) {
          console.error("Failed to delete entry.", {
            model: model.modelId,
            id: item.id
          });
          return this.response.error(new Error(`Failed to delete entry "${item.id}". Cannot continue.`));
        }
        lastDeletedId = item.entryId;
      }
      hasMoreItems = metaHasMoreItems;
    } while (hasMoreItems);
    /**
     * Let's do one more check. If there are items, continue the task with 5 seconds delay.
     */
    const {
      items
    } = await this.context.cms.storageOperations.entries.list(model, {
      limit: 1,
      where: {
        latest: true
      }
    });
    if (items.length > 0) {
      console.log("There are still items to be deleted. Continuing the task.");
      return this.response.continue({
        ...input
      }, {
        seconds: 5
      });
    }
    let hasMoreFolders = false;
    do {
      const [items, meta] = await this.context.aco.folder.list({
        where: {
          type: `cms:${model.modelId}`
        },
        limit: 1000
      });
      for (const item of items) {
        try {
          await this.context.aco.folder.delete(item.id);
        } catch (ex) {
          console.error(`Failed to delete folder "${item.id}".`, ex);
          return this.response.error(ex);
        }
      }
      hasMoreFolders = meta.hasMoreItems;
    } while (hasMoreFolders);

    /**
     * When there is no more records to be deleted, let's delete the model, if it's not a plugin.
     */
    await this.removeBeingDeleted(model);
    if (model.isPlugin) {
      return this.response.done();
    }
    try {
      await this.context.cms.deleteModel(model.modelId);
    } catch (ex) {
      const message = `Failed to delete model "${model.modelId}".`;
      console.error(message);
      return this.response.error(ex);
    }
    return this.response.done();
  }
  async getModel(modelId) {
    const model = await this.context.cms.getModel(modelId);
    if (!model) {
      throw new Error(`Model "${modelId}" not found.`);
    }
    return model;
  }
  async removeBeingDeleted(model) {
    const key = (0, _store.createStoreKey)(model);
    await this.context.db.store.removeValue(key);
  }
}
exports.DeleteModelRunner = DeleteModelRunner;
const createDeleteModelRunner = params => {
  return new DeleteModelRunner(params);
};
exports.createDeleteModelRunner = createDeleteModelRunner;

//# sourceMappingURL=DeleteModelRunner.js.map