"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDeleteModelGraphQl = void 0;
var _zod = _interopRequireDefault(require("zod"));
var _api = require("@webiny/api");
var _apiHeadlessCms = require("@webiny/api-headless-cms");
var _confirmation = require("../helpers/confirmation");
var _handlerGraphql = require("@webiny/handler-graphql");
var _utils = require("@webiny/utils");
const deleteValidation = _zod.default.object({
  modelId: _zod.default.string(),
  confirmation: _zod.default.string()
}).superRefine((value, context) => {
  if ((0, _confirmation.validateConfirmation)(value)) {
    return;
  }
  context.addIssue({
    code: _zod.default.ZodIssueCode.custom,
    message: `Confirmation input does not match.`,
    fatal: true,
    path: ["confirmation"]
  });
}).readonly();
const cancelValidation = _zod.default.object({
  modelId: _zod.default.string()
}).readonly();
const getValidation = _zod.default.object({
  modelId: _zod.default.string()
}).readonly();
const createDeleteModelGraphQl = () => {
  const contextPlugin = new _api.ContextPlugin(async inputContext => {
    const ready = await (0, _apiHeadlessCms.isHeadlessCmsReady)(inputContext);
    if (!ready || !inputContext.cms.MANAGE) {
      return;
    }
    const plugin = new _apiHeadlessCms.CmsGraphQLSchemaPlugin({
      typeDefs: /* GraphQL */`
                enum DeleteCmsModelTaskStatus {
                    running
                    done
                    error
                    canceled
                }
                type DeleteCmsModelTask {
                    id: ID!
                    status: DeleteCmsModelTaskStatus!
                    deleted: Int!
                    total: Int!
                }

                type GetDeleteCmsModelProgressResponse {
                    data: DeleteCmsModelTask
                    error: CmsError
                }

                type FullyDeleteCmsModelResponse {
                    data: DeleteCmsModelTask
                    error: CmsError
                }

                type CancelDeleteCmsModelResponse {
                    data: DeleteCmsModelTask
                    error: CmsError
                }

                extend type CmsContentModel {
                    isBeingDeleted: Boolean!
                }

                extend type Query {
                    getDeleteModelProgress(modelId: ID!): GetDeleteCmsModelProgressResponse!
                    listContentModels(
                        includeBeingDeleted: Boolean = false
                    ): CmsContentModelListResponse
                }

                extend type Mutation {
                    fullyDeleteModel(
                        modelId: ID!
                        confirmation: String!
                    ): FullyDeleteCmsModelResponse!
                    cancelFullyDeleteModel(modelId: ID!): CancelDeleteCmsModelResponse!
                }
            `,
      resolvers: {
        CmsContentModel: {
          isBeingDeleted: async (model, _, context) => {
            try {
              return await context.cms.isModelBeingDeleted(model.modelId);
            } catch (ex) {
              console.error(ex);
            }
            return true;
          }
        },
        Query: {
          getDeleteModelProgress: async (_, args, context) => {
            return (0, _handlerGraphql.resolve)(async () => {
              const input = getValidation.safeParse(args);
              if (input.error) {
                throw (0, _utils.createZodError)(input.error);
              }
              return await context.cms.getDeleteModelProgress(input.data.modelId);
            });
          }
        },
        Mutation: {
          fullyDeleteModel: async (_, args, context) => {
            return (0, _handlerGraphql.resolve)(async () => {
              const input = deleteValidation.safeParse(args);
              if (input.error) {
                throw (0, _utils.createZodError)(input.error);
              }
              return await context.cms.fullyDeleteModel(input.data.modelId);
            });
          },
          cancelFullyDeleteModel: async (_, args, context) => {
            return (0, _handlerGraphql.resolve)(async () => {
              const input = cancelValidation.safeParse(args);
              if (input.error) {
                throw (0, _utils.createZodError)(input.error);
              }
              return await context.cms.cancelFullyDeleteModel(input.data.modelId);
            });
          }
        }
      },
      resolverDecorators: {
        ["Query.listContentModels"]: [(0, _handlerGraphql.createResolverDecorator)(resolver => async (parent, args, context, info) => {
          const result = await resolver(parent, args, context, info);
          if (result.error || !Array.isArray(result.data)) {
            return result;
          }
          if (args?.includeBeingDeleted !== false) {
            return result;
          }
          const listed = result.data;
          try {
            const beingDeletedList = await context.cms.listModelsBeingDeleted();
            return new _handlerGraphql.Response(listed.filter(model => {
              if (!model?.modelId) {
                return false;
              } else if (beingDeletedList.some(item => item.modelId === model.modelId)) {
                return false;
              }
              return true;
            }));
          } catch (ex) {
            return new _handlerGraphql.ErrorResponse(ex);
          }
        })]
      }
    });
    plugin.name = "headless-cms.graphql.fullyDeleteModel";
    inputContext.plugins.register(plugin);
  });
  contextPlugin.name = "headless-cms.context.createDeleteModelGraphQl";
  return contextPlugin;
};
exports.createDeleteModelGraphQl = createDeleteModelGraphQl;

//# sourceMappingURL=index.js.map